--[[----------------------------------
--  MultiSelect widget for AceGUI-3.0
--  Written by Shirokuma
--]]----------------------------------


--[[-----------------
-- AceGUI
--]]-----------------
local AceGUI = LibStub("AceGUI-3.0")

--[[-----------------
-- Lua APIs
--]]-----------------
local format, pairs, tostring = string.format, pairs, tostring

--[[-----------------
-- WoW APIs
--]]-----------------
local CreateFrame, UIParent = CreateFrame, UIParent

--[[-----------------
-- Frame Elements
--]]-----------------
local FrameBackdrop = {
	bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
	tile = true, tileSize = 16, edgeSize = 16,
	insets = { left = 3, right = 3, top = 3, bottom = 3 }
}

local LabelBackdrop = {
	bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
	edgeFile = "",
	tile = true, tileSize = 16, edgeSize = 12,
	insets = { left = 0, right = 0, top = 1, bottom = 1 }
}


--[[-----------------
-- Widget Info
--]]-----------------
local widgetType = "MultiSelect"
local widgetVersion = 2


--[[-----------------
-- Event Code
--]]-----------------
local function Label_OnEnter(label)
	local self = label.obj
	local value = label
	self:Fire("OnLabelEnter", value)
end

local function Label_OnLeave(label)
	local self = label.obj
	local value = label
	self:Fire("OnLabelEnter", value)
end

local function Label_OnClick(label)
	local self = label.obj
	local value = label
	self:Fire("OnLabelClick", value)
	AceGUI:ClearFocus()
end

local function OnSizeChanged(self, width, height)
	self.obj.labelframe:SetWidth(width)
end


--[[-----------------
-- MultiSelect Code
--]]-----------------
do
	local methods = {
		["OnAcquire"] = function (self)  -- set up the default size
			self:SetWidth(200)
			self:SetHeight(400)
		end,
		
		["SetLabel"] = function (self, text)  -- sets the multiselect label text
			self.label:SetText(text)
		end,
		
		["SetMultiSelect"] = function (self, value)  -- set if multiple values can be selected simultaneously
			self.multiselect = value
		end,
		
		["AddItem"] = function (self, str)  -- add an item (create a new item label object)
			local label = CreateFrame("Button", nil, self.labelframe)
			label.selected = false
			label.obj = self
			label:SetHeight(18)
			label:SetPoint("TOPLEFT", self.labelframe, "TOPLEFT", 0, -(getn(self.labels) * 18))
			label:SetPoint("TOPRIGHT", self.labelframe, "TOPRIGHT", 0, -(getn(self.labels) * 18))
			
			label:SetBackdrop(LabelBackdrop)
			label:SetBackdropColor(0,0,0,0)
			label:SetBackdropBorderColor(0,0,0,0)
			self.labels[getn(self.labels) + 1] = label
			self.labelframe:SetHeight(getn(self.labels) * 18)
			
			local text = label:CreateFontString(nil,"OVERLAY","GameFontNormalSmall")
			text:SetJustifyH("LEFT")
			text:SetPoint("TOPLEFT",label,"TOPLEFT",5,0)
			text:SetPoint("BOTTOMRIGHT",label,"BOTTOMRIGHT",-5,0)
			text:SetText(str)
			label.text = text
			
			label:SetScript("OnEnter", function(this)
				this:SetBackdropColor(1,0.9,0,1)
				Label_OnEnter(this)
			end)
			label:SetScript("OnLeave", function(this)
				this:SetBackdropBorderColor(0,0,0,0)
				if this.selected then
					this:SetBackdropColor(0.9,0.7,0,1)
				else
					this:SetBackdropColor(0,0,0,0)
				end
			end)
			label:SetScript("OnClick", function(this)
				if not this.selected then
					this:SetBackdropColor(0.9,0.7,0,1)
					this.selected = true
					if not self.multiselect then
						for index, items in pairs(self.labels) do
							if self.labels[index] ~= this and self.labels[index].selected then
								self.labels[index].selected = false
								self.labels[index]:SetBackdropColor(0,0,0,0)
							end
						end
					end
				else
					this.selected = false
				end
				Label_OnClick(this)
			end)
		end,
		
		["GetItem"] = function (self, text)  -- find an object based on the text parameter
			for _, value in pairs(self.labels) do
				if value.text:GetText() == text then
					return value
				end
			end
			return nil
		end,
		
		["GetText"] = function (self, value)  -- get the text of a label object
			for _,item in pairs(self.labels) do
				if value == item then
					return item.text:GetText()
				end
			end
			return nil
		end,
		
		["SetText"] = function (self, value, text)  -- set the text of a label object
			for _, item in pairs(self.labels) do
				if value == item then
					value.text:SetText(text)
				end
			end
		end,
		
		["IsSelected"] = function (self, value)  -- return if the label object is currently selected
			for _, item in pairs(self.labels) do
				if value == item then
					return item.selected
				end
			end
			return nil
		end,
		
		["GetSelected"] = function (self)  -- return a table of the currently selected label objects
			local selectedList = {}
			for _, item in pairs(self.labels) do
				if item.selected then
					table.insert(selectedList, item)
				end
			end
			return selectedList
		end,
			
		["SetItemList"] = function (self, list)  -- create new labels from a list of strings
			for _,item in pairs(self.labels) do
				item:Hide()
				item:ClearAllPoints()
			end
			
			self.labels = {}
			
			if list then
				for _,item in pairs(list) do
					self:AddItem(item)
				end
			end
		end,

		["RemoveItem"] = function (self, item)  -- delete an item
			local function RedrawFrame()
				for index,value in pairs(self.labels) do
					value:SetPoint("TOPLEFT", self.labelframe, "TOPLEFT", 0, (-(index-1) * 18))
					value:SetPoint("TOPRIGHT", self.labelframe, "TOPRIGHT", 0,(-(index-1) * 18))
				end
			end
			
			for index, value in pairs(self.labels) do
				if value == item then
					table.remove(self.labels, index)
					item:Hide()
					item:ClearAllPoints()
					RedrawFrame()
				end
			end
		end,
		
		["SetSelected"] = function (self, item, value)
			if item then	
				if value then
					if not self.multiselect then  -- test
						for _, value in pairs(self.labels) do
							value.selected = false
							value:SetBackdropColor(0,0,0,0)
						end
					end
					item.selected = true
					item:SetBackdropColor(0.9,0.7,0,1)
				else
					item.selected = false
					item:SetBackdropColor(0,0,0,0)
				end
			else
				for _, value in pairs(self.labels) do
					value.selected = false
					value:SetBackdropColor(0,0,0,0)
				end
			end
		end
	}
	
	local function Constructor()  -- widget constructor
		local frame = CreateFrame("Frame", nil, UIParent)
		local self = {}
		local labels = {}
		
		self.type = widgetType
		self.frame = frame
		self.backdrop = backdrop
		self.labels = {}
		self.multiselect = true
		frame.obj = self
		
		local label = frame:CreateFontString(nil,"OVERLAY","GameFontNormalSmall")
			label:SetJustifyH("LEFT")
			label:SetPoint("TOPLEFT", 0, 0)
			label:SetPoint("TOPRIGHT", 0, 0)
			label:SetHeight(10)
			label:SetText("MultiSelect")
			self.label = label

		local backdrop = CreateFrame("Frame", nil, frame)
			backdrop:SetBackdrop(FrameBackdrop)
			backdrop:SetBackdropColor(0, 0, 0)
			backdrop:SetBackdropBorderColor(0.4, 0.4, 0.4)
			backdrop:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, -10)
			backdrop:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -16, 0)
		
		local scrollframe = CreateFrame("ScrollFrame", format("%s@%s@%s", widgetType, "ScrollFrame", tostring(self)), frame, "UIPanelScrollFrameTemplate")
			local scrollbar = _G[scrollframe:GetName() .. "ScrollBar"]
				scrollbar:ClearAllPoints()
				scrollbar:SetPoint("TOP", label, "BOTTOM", 0, -19)
				scrollbar:SetPoint("BOTTOM", frame, "BOTTOM", 0, 19)
				scrollbar:SetPoint("RIGHT", frame, "RIGHT", 0, 0)
				
			scrollframe:SetPoint("TOPLEFT", backdrop, "TOPLEFT", 10, -6)
			scrollframe:SetPoint("BOTTOMRIGHT", backdrop, "BOTTOMRIGHT", -10, 6)
			scrollframe:SetScript("OnSizeChanged", OnSizeChanged)
			scrollframe.obj = self
			self.scrollframe = scrollframe
		
		local labelframe = CreateFrame("Frame", nil, scrollframe)
			labelframe:SetAllPoints()
			labelframe.obj = self
			scrollframe:SetScrollChild(labelframe)
			self.labelframe = labelframe

		for method, func in pairs (methods) do
			self[method] = func
		end
		
		AceGUI:RegisterAsWidget(self)
		return self
	end
	AceGUI:RegisterWidgetType(widgetType, Constructor, widgetVersion)
end